local Critters = Critters
local L = LibStub("AceLocale-3.0"):GetLocale("Critters", false)

Critters.Option = {	
	type="group",
	width = 'fill',
	handler = Critters,
	args={
		EnableMacro = {
			type = 'toggle',
			order = 1,
			name = L['Enable Macro'],
			desc = L['EnableMacrodesc'],
			get = function() 
				return Critters.db.char.enablemacro
			end,
			set = function(info, key) 
				Critters.db.char.enablemacro = key
				if key then
					if GetMacroIndexByName("Critters") == 0 then
						CreateMacro("Critters", 1, '')
						Critters:UpdateMacro()
					end
					Critters:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
					Critters:RegisterEvent("PLAYER_REGEN_DISABLED")
					Critters:RegisterEvent("PLAYER_REGEN_ENABLED")
				else
					DeleteMacro("Critters")
					Critters:UnregisterEvent("UNIT_SPELLCAST_SUCCEEDED")
					Critters:UnregisterEvent("PLAYER_REGEN_DISABLED")
					Critters:UnregisterEvent("PLAYER_REGEN_ENABLED")
				end
			end,
		},
		clickreagents = {
			type = 'toggle',
			order = 2,
			name = L['Macro Reagents'],
			desc = L['Macro Reagentsdesc'],
			get = function() 
				return Critters.db.char.clickreagents
			end,
			set = function(info, key) 
				Critters.db.char.clickreagents = key
			end,
			disabled = function()
				return not Critters.db.char.enablemacro
			end,
		},
		EnableAutosummon = {
			type = 'toggle',
			order = 3,
			name = L['Auto Call Companion'],
			desc = L['Auto Call Companiondesc'],
			get = function() 
				return Critters.db.char.enableautosummon
			end,
			set = function(info, key) 
				Critters.db.char.enableautosummon = key
				if key then
					Critters:RegisterEvent("PLAYER_ENTERING_WORLD")
					Critters:RegisterEvent("COMPANION_UPDATE")
					Critters:RegisterEvent("PLAYER_UNGHOST")
					Critters:RegisterEvent("PLAYER_CONTROL_GAINED")
					Critters:RegisterEvent("ZONE_CHANGED_NEW_AREA")
				else
					Critters:UnregisterEvent("PLAYER_ENTERING_WORLD")
					Critters:UnregisterEvent("COMPANION_UPDATE")
					Critters:UnregisterEvent("PLAYER_UNGHOST")
					Critters:UnregisterEvent("PLAYER_CONTROL_GAINED")
					Critters:UnregisterEvent("ZONE_CHANGED_NEW_AREA")
				end
			end,
		},
		autoreagents = {
			type = 'toggle',
			order = 4,
			name = L['Auto Reagents'],
			desc = L['Auto Reagentsdesc'],
			get = function() 
				return Critters.db.char.autoreagents
			end,
			set = function(info, key) 
				Critters.db.char.autoreagents = key
			end,
			disabled = function()
				return not Critters.db.char.enableautosummon
			end,
		},
	},
}

function Critters:UpdateOptions()
	local mounts = {}
	for k in pairs(self.db.char.Critters) do
		mounts[#mounts + 1] = GetSpellInfo(k)
	end
	table.sort(mounts)
	for i, mountname in ipairs(mounts) do
		self.Option.args[tostring(i)] = self:CreateRange(mountname, i)
	end
end

function Critters:GetRange(info)
	return self.db.char.Critters[self.ReverseLookup[info.option.name]]
end

function Critters:SetRange(info, num)
	self.db.char.Critters[self.ReverseLookup[info.option.name]] = num
end

function Critters:CreateRange(mountname,ordernum)
	local Range = {
			type = 'range',
			name = mountname,
			order = ordernum+100,
			--width = 'full',
			min = 0,
			max = 10,
			step = 1,
			get = 'GetRange',
			set = 'SetRange',
		}
	return Range
end

Critters.optframe = {}
LibStub("AceConfig-3.0"):RegisterOptionsTable('Critters', Critters.Option)
Critters.optframe.Critters = LibStub("AceConfigDialog-3.0"):AddToBlizOptions('Critters', 'Critters')
-- function addapted from the RegisterModuleOptions function in Mapster
function Critters:RegisterModuleOptions(name, optionTbl, displayName)
	local cname = "Critters"..name
	LibStub("AceConfigRegistry-3.0"):RegisterOptionsTable(cname, optionTbl)
	self.optframe[name] = LibStub("AceConfigDialog-3.0"):AddToBlizOptions(cname, displayName, "Critters")
end

do -- combat macro ahoy
	local db = {
		["char"] = L['char'],
		["class"] = L['class'],
		["race"] = L['race'],
		['global'] = L['global'],
	}
	
	local MacroOptions = {
		type="group",
		width = 'fill',
		handler = Critters,
		childGroups = 'tab',
		args={
			Enable={
				type = 'toggle',
				order = 1,
				name = L['Enable'],
				desc = L['Enabledesc'],
				get = 'OGet',
				set = 'OSet',
				disabled = function()
					if not Critters.db.char.enablemacro then
						return true
					end
				end,
			},
			linebreak0 = {
				name = '',
				order = 5,
				type="description",
			},
			select={
				name=L['Select'],
				type="select",
				order = 10,
				values = function()
					local v = {}
					for k in pairs(Critters.db.global.macros) do
						if k == 'Default' then
							v[k] = L[k]
						else
							v[k] = k
						end
					end
					v['new'] = L["New"]
					return v
				end,
				get = 'OGet',
				set = 'OSet',
				disabled = 'ODisabled',
			},
			name={
				name=L['Name'],
				type="input",
				order = 20,
				get = 'OGet',
				set = 'OSet',
				validate = function(info,text)
					if Critters.db.global.macros[text] then
						Critters:Print(string.format(L["NameError"], text))
						return string.format(L["NameError"], text)
					else
						return true
					end
				end,
				disabled = 'ODisabled',
			},
			macro={
				name=ENTER_MACRO_LABEL,
				desc=L["FAQdesc"],
				type="input",
				multiline  = true,
				width = 'full',
				order = 30,
				get = 'OGet',
				set = 'OSet',
				validate = 'Ovalidate',
				disabled = 'ODisabled',
			},
			preview={
				type = 'toggle',
				order = 40,
				name = L['Preview'],
				desc = L['Toggle Preview'],
				get = 'OGet',
				set = 'OSet',
				disabled = 'ODisabled',
			},
			linebreak1 = {
				name = '',
				order = 45,
				type="description",
			},
			set = {
				name=L['Set Macro'],
				desc=L['Will set the macro to the selected one.\nWill also save the macro.'],
				type="execute",
				order = 50,
				func = 'Ofunc',
				disabled = 'ODisabled',
			},
			reset = {
				name=L['Reset Macro'],
				desc=L['Will discard all changes and reset the macro to the previous one.'],
				type="execute",
				order = 60,
				func = 'Ofunc',
				disabled = 'ODisabled',
			},
			delete = {
				name=L['Delete Macro'],
				desc=L['Will delete the selected macro and revert to the default macro.'],
				type="execute",
				order = 70,
				func = 'Ofunc',
				disabled = 'ODisabled',
			},
			linebreak2 = {
				name = '',
				order = 80,
				type="description",
			},
			db={
				name=L['db'],
				type="select",
				order = 90,
				values = db,
				get = 'OGet',
				set = 'OSet',
				disabled = 'ODisabled',
			},
		},
	}
	
	function Critters:OGet(info)
		if info[#info] == 'select' or info[#info] == 'name' then
			if not self.name then
				self.name = self.db[self.db.char.cmloc].cm
			end
			return self.name
		elseif info[#info] == 'db' then
			return self.db.char.cmloc
		elseif info[#info] == 'macro' then
			if not self.macro then
				self.macro = self.db.global.macros[self.db[self.db.char.cmloc].cm]
			end
			if self.preview then
				return self:ParseMacro(self.macro, true)
			end
			return self.macro
		elseif info[#info] == 'preview' then
			return self.preview
		elseif info[#info] == 'Enable' then
			return self.db.char.cmenabled
		end
	end

	function Critters:OSet(info, key)
		if info[#info] == 'select' then
			if key == 'new' then
				self.name = ''
				self.macro = ''
				self.preview = nil
			else
				self.name = key
				self.macro = self.db.global.macros[key]
			end
		elseif info[#info] == 'name' then
			self.name = key
		elseif info[#info] == 'db' then
			if not self.db[key] then
				self.db[key] = self.db[self.db.char.cmloc].cm
			end
			self.db.char.cmloc = key
		elseif info[#info] == 'macro' then
			if not self.locked[self.name] and not self.preview then
				self.macro = key
			end
		elseif info[#info] == 'preview' then
			self.preview = key
		elseif info[#info] == 'Enable' then
			self.db.char.cmenabled = key
		end
	end

	function Critters:Ofunc(info)
		if info[#info] == 'set' then
			self.db.global.macros[self.name] = self.macro
			self.db[self.db.char.cmloc].cm = self.name
		elseif info[#info] == 'reset' then
			self.name = nil
			self.macro = nil
		elseif info[#info] == 'delete' then
			if self.db[self.db.char.cmloc].cm == self.name then
				self.db[self.db.char.cmloc].cm = 'Default'
			end
			self.db.global.macros[self.name] = nil
			self.name = nil
			self.macro = nil
		end
	end

	function Critters:Ovalidate(info, macro)
		for i=1,10 do
			if string.len(self:ParseMacro(macro, true)) > 255 then
				self:Print(L["Macro length error"])
				return L["Macro length error"]
			end
		end
		return true
	end

	function Critters:ODisabled(info)
		if not self.db.char.enablemacro then
			return true
		end
		if self.db.char.cmenabled then
			if info[#info] == 'name' or info[#info] == 'delete' then
				if self.locked[self.name] then
					return true
				end
			elseif info[#info] == 'macro' then
				if self.locked[self.name] or self.preview then
					return true
				end
			elseif info[#info] == 'set' then
				if self.name == '' or self.macro == '' or self.db[self.db.char.cmloc].cm == self.name then
					return true
				end
			elseif info[#info] == 'reset' then
				if self.db[self.db.char.cmloc].cm == self.name then
					return true
				end
			end
			return nil
		else
			return true
		end
	end
	Critters:RegisterModuleOptions(L['CombatMacro'], MacroOptions, L['CombatMacro'])
end


